﻿using System.ComponentModel;
using System.Drawing;
using System.Text;
using Cassette.Extensions;
using JetBrains.Annotations;
using Remora.Commands.Attributes;
using Remora.Commands.Groups;
using Remora.Discord.API.Abstractions.Objects;
using Remora.Discord.API.Abstractions.Rest;
using Remora.Discord.API.Objects;
using Remora.Discord.Commands.Feedback.Messages;
using Remora.Discord.Commands.Feedback.Services;
using Remora.Discord.Extensions.Embeds;
using Remora.Discord.Extensions.Formatting;
using Remora.Results;

namespace Cassette.Commands;

/// <summary>
///     A <see cref="CommandGroup"/> that shows information about the bot.
/// </summary>
public sealed class BotCommandGroup(
    IFeedbackService feedbackService,
    IDiscordRestUserAPI userApi) : CommandGroup
{
    private const string RepositoryUrl = "https://git.mctaylors.ru/mctaylors/Cassette";

    [Command("about")]
    [Description("Who developed this bot?")]
    [UsedImplicitly]
    public async Task<Result> AboutCommandAsync()
    {
        var botResult = await userApi.GetCurrentUserAsync();
        if (!botResult.IsDefined(out var bot))
        {
            return Result.FromError(botResult);
        }

        var description = new StringBuilder()
            .Append("Cassette is a Discord bot written by ")
            .Append(Markdown.Hyperlink("@mctaylors", "https://git.mctaylors.ru/mctaylors"))
            .AppendLine(" that plays your most favourite music in your most visited voice channels.")
            .AppendLine("Powered by Remora.Discord & Lavalink4NET.");

        var embed = new EmbedBuilder()
            .WithUserAuthor($"About {bot.Username}", bot)
            .WithDescription(description.ToString())
            .WithColour(Color.Teal)
            .Build();

        var button = new ButtonComponent(
            ButtonComponentStyle.Link,
            "Open Repository",
            URL: RepositoryUrl
        );

        return await feedbackService.SendContextualEmbedResult(embed,
            new FeedbackMessageOptions(MessageComponents: new[]
            {
                new ActionRowComponent(new[] { button })
            }));
    }
}
